from pathlib import Path
import json
from .character_sets.hiragana import *


class CharacterStatistics:
    def __init__(self, file):
        self.file = file

    def load(self):
        if Path(self.file).is_file():
            with open(self.file, "r") as f:
                data = json.load(f)
        else:
            data = {}
        self.data = data

    def update_character(self, character, correct, timedelta, wrong_char_romaji=None):
        if character in self.data:
            self.data[character]["correct"] += correct
            self.data[character]["total"] += 1
            self.data[character]["times"].append(timedelta)
            self.data[character]["average_time"] = round(
                sum(self.data[character]["times"]) / len(self.data[character]["times"]),
                2,
            )
            self.data[character]["accuracy"] = round(
                self.data[character]["correct"] / self.data[character]["total"] * 100, 2
            )
        else:
            self.data[character] = {
                "correct": correct,
                "total": 1,
                "times": [timedelta],
                "average_time": timedelta,
                "accuracy": correct / 1 * 100,
                "mixups": {},
            }

        wrong_char = reverse_hiragana.get(wrong_char_romaji, None)
        if wrong_char:
            if wrong_char in self.data[character]["mixups"]:
                self.data[character]["mixups"][wrong_char] += 1
            else:
                self.data[character]["mixups"][wrong_char] = 1

    def _get_longest_delay_characters(self):
        print()
        print("┌─────────┐")
        print("│ SLOWEST │")
        print("└─────────┘")
        for character, stats in sorted(
            self.data.items(), key=lambda x: x[1]["average_time"], reverse=True
        )[:5]:
            print(
                f"{character}\t{Hiragana.hiragana.value[character]}\t{stats['average_time']}"
            )

    def _get_least_accurate_characters(self):
        print("┌─────────────────┐")
        print("│ LOWEST ACCURACY │")
        print("└─────────────────┘")

        for character, stats in sorted(
            self.data.items(), key=lambda x: x[1]["accuracy"]
        )[:5]:
            mixups = []
            for mixup_char in sorted(stats["mixups"].keys(), reverse=True)[:3]:
                mixups.append(mixup_char)
            print(
                f"{character}\t{Hiragana.hiragana.value[character]}\t{round(stats['accuracy'])}\t{', '.join(mixups)}"
            )

    def print_stats(self):
        self._get_longest_delay_characters()
        self._get_least_accurate_characters()

    def write(self):
        with open(self.file, "w") as f:
            json.dump(self.data, f)



from sys import exit
from argparse import ArgumentParser
from .version import __version__
from datetime import datetime
from signal import signal, SIGINT
import random
from .character_statistics import CharacterStatistics
from .character_sets.hiragana import Hiragana


def main(args):
    character_set = Hiragana[args.character_set].value
    character_statistics = CharacterStatistics(args.statistics_file)
    character_statistics.load()

    def signal_handler(*args):
        character_statistics.print_stats()
        character_statistics.write()
        exit(0)

    signal(SIGINT, signal_handler)

    while True:
        before_answer = datetime.now()
        character = random.choice(list(character_set))
        answer = input(f"{character} ")
        after_answer = datetime.now()
        timedelta = round((after_answer - before_answer).total_seconds(), 2)
        if answer == character_set[character]:
            print("✓")
            character_statistics.update_character(character, 1, timedelta)
        else:
            print(f"✗ {character_set[character]}")
            character_statistics.update_character(character, 0, timedelta, answer)


def cli():
    parser = ArgumentParser(description="Practice Japanese characters")

    parser.add_argument(
        "-c",
        "--character-set",
        type=str,
        choices=Hiragana._member_names_,
        help="Which character set to practice",
        default="hiragana",
    )
    parser.add_argument(
        "-s",
        "--statistics-file",
        type=str,
        help="File to save character statistics to",
        default="character_stats.json",
    )

    args = parser.parse_args()
    main(args)



from setuptools import setup, find_packages
import os
import sys

_here = os.path.abspath(os.path.dirname(__file__))

if sys.version_info[0] < 3:
    with open(os.path.join(_here, "README.md")) as f:
        long_description = f.read()
else:
    with open(os.path.join(_here, "README.md"), encoding="utf-8") as f:
        long_description = f.read()

version = {}
with open(os.path.join(_here, "practice_japanese", "version.py")) as f:
    exec(f.read(), version)

with open("requirements.txt") as f:
    requirements = f.readlines()

setup(
    name="practice_japanese",
    version=version["__version__"],
    description="Practice Japanese character sets.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Heather Ward",
    author_email="heather.ward13@gmail.com",
    url="https://github.com/hkeward/practice_japanese",
    license="gpl-2.0",
    packages=find_packages(),
    install_requires=requirements,
    include_package_data=True,
    entry_points={
        "console_scripts": ["practice_japanese=practice_japanese.command_line:cli"]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: Unix",
    ],
)
